program AVR_I2Cctrl;

{ $BOOTRST $0F000}         {Reset Jump to $0F000}
{$NOSHADOW}
{ $W+ Warnings}            {Warnings off}

Device = mega128, VCC=5;

Import SysTick, SerPort, LCDmultiPort, TWImaster;

From System Import ;

Define
        ProcClock   = 16000000;        {Hertz}
        SysTick     = 10;             {msec}
        StackSize   = $0064, iData;
        FrameSize   = $0064, iData;
        SerPort     = 19200, Stop2;    {Baud, StopBits|Parity}
        RxBuffer    = 40, iData;
        TxBuffer    = 40, iData;
        TWIpresc    = TWI_BR400;
        LCDmultiPort = I2C_TWI;
        LCDrows_M    = 4;             {rows}
        LCDcolumns_M = 16;             {columns per line}
        LCDtype_M    = 44780;

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }

type


{--------------------------------------------------------------}
{ Const Declarations }
const
  ADS7823    : byte = $48;
  LM75       : byte = $49;
  PCA9532    : byte = $60;


{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
var
  bb, lp   : byte;
  ii       : integer;

  temp     : word;
  curErrT  : integer;
  errIntT  : integer;
  declValT : integer;
  outpValT : byte;
  FilterT  : AVfilter[0..7] of integer;

  speed    : word;
  curErrM  : integer;
  errIntM  : integer;
  declValM : integer;
  outpValM : byte;
  FilterM  : AVfilter[0..7] of integer;

{--------------------------------------------------------------}
{ functions }



{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  EnableInts;
  LCDsetup_M(LCD_m1);
  LCDcursor_M(LCD_m1, false, false);
  Write(LCDOut_M,  'E-LAB I2C Loop');
  LCDxy_M(LCD_m1, 0, 1);
  Write(LCDOut_M,  'Demo Program');

  LCDclrLine_M(LCD_m1, 2);
  LCDxy_M(LCD_m1, 0, 2);
  if TWIstat(ADS7823) then
    Write(LCDOut_M,  'ADS7823 found');
  else
    Write(LCDOut_M,  'ADS7823 not found');
  endif;
  mDelay(2000);

  LCDclrLine_M(LCD_m1, 2);
  LCDxy_M(LCD_m1, 0, 2);
  if TWIstat(LM75) then
    Write(LCDOut_M,  'LM75 found');
  else
    Write(LCDOut_M,  'LM75 not found');
  endif;
  mDelay(2000);

  LCDclrLine_M(LCD_m1, 2);
  LCDxy_M(LCD_m1, 0, 2);
  if TWIstat(PCA9532) then
    Write(LCDOut_M,  'PCA9532 found');
  else
    Write(LCDOut_M,  'PCA9532 not found');
  endif;
  mDelay(2000);

  TWIout(PCA9532, 2, 0);                        // presc reg0 = 0   = max frequency
  TWIout(PCA9532, 4, 0);                        // presc reg1 = 0   = max frequency
  TWIout(PCA9532, 3, 127);                      // pwm reg0   = 127 = 50% duty cycle
  TWIout(PCA9532, 5, 127);                      // pwm reg1   = 127 = 50% duty cycle
  TWIout(PCA9532, 6, $FE);                      // led reg0   = 256 = Led0 -> PWM0, Led1..3 -> PWM1

  TWIout(LM75, 0);                              // prepare read register 0 = temperatur

  TWIinp(LM75, temp);
  ii:= integer(500) - integer(temp);            // build difference = error  for 50% startup value
  PresetAVfilter(FilterT, ii);                  // set initial values
  errIntT:= 255 * 50;

  TWIout(ADS7823, 0);                           // start operation
  TWIinp(ADS7823, speed);
  speed:= swap(speed);
  ii:= integer(1500) - integer(speed);          // build difference = error  for 50% startup value
  PresetAVfilter(FilterM, ii);                  // set initial values
  errIntM:= 1500;                               //

  lp:= 10;
  loop
    mDelay(50);
    TWIout(ADS7823, 0);                           // start conversion
    TWIinp(ADS7823, speed);
    speed:= swap(speed);
    TWIinp(LM75, temp);
    temp:= swap(temp) shr 7;
    temp:= MulDivInt(temp, 50, 10);

    ii:= integer(500) - integer(temp);          // build difference = error
    curErrT:= ii;                               // = P
    errIntT:= ii + errIntT;                     // update the error integral = I
    SetAVfilter(FilterT, curErrT*2);            // set actual error
    declValT:= DeclAVfilter(FilterT);           // get the error declination = D
    //           P   +    I   +    D
    ii:= (curErrT*2) + (errIntT div 100) + declValT;             // build the sum of P I D
    ii:= WithIn(0, ii, 255);
    outpValT:= byte(ii);
    TWIout(PCA9532, 5, outpValT);               // pwm reg1   = 127 = 50% duty cycle

    ii:= integer(1500) - integer(speed);        // build difference = error
    curErrM:= ii;                               // = P
    errIntM:= ii + errIntM;                     // update the error integral = I
    SetAVfilter(FilterM, curErrM*2);            // set actual error
    declValM:= DeclAVfilter(FilterM);           // get the error declination = D
    //           P   +    I   +    D
    ii:= (curErrM div 25) + (errIntM div 60) + (declValM div 50);             // build the sum of P I D
    ii:= WithIn(0, ii, 255);
    outpValM:= byte(ii);
    TWIout(PCA9532, 3, outpValM);               // pwm reg0   = 127 = 50% duty cycle

    dec(lp);
    if lp = 0 then
      lp:= 10;
      LCDxy_M(LCD_m1, 0, 0);
      LCDclrEOL_M(LCD_m1);
      Write(LCDOut_M, 'A: '+IntToStr(temp:4:1));
      LCDxy_M(LCD_m1, 8, 0);
      Write(LCDOut_M, 'N: '+IntToStr(500:4:1));
      LCDxy_M(LCD_m1, 0, 1);
      LCDclrEOL_M(LCD_m1);
      Write(LCDOut_M, 'P: '+IntToStr(curErrT));
      LCDxy_M(LCD_m1, 0, 2);
      LCDclrEOL_M(LCD_m1);
      Write(LCDOut_M, 'I: '+IntToStr(errIntT));
      LCDxy_M(LCD_m1, 0, 3);
      LCDclrEOL_M(LCD_m1);
      Write(LCDOut_M, 'D: '+IntToStr(declValT:4));
      LCDxy_M(LCD_m1, 9, 3);
      Write(LCDOut_M, 'O: '+ByteToStr(outpValT:3));

      LCDxy_M(LCD_m1, 9, 2);
      Write(LCDOut_M, 'V: '+IntToStr(speed:4));
      LCDxy_M(LCD_m1, 9, 1);
      Write(LCDOut_M, 'M: '+ByteToStr(outpValM:3));
    endif;
  endloop;
end AVR_I2Cctrl.

